<?php

class Main extends Controller {
	
	//список состояний приложения и соответствующих картинок
	private $states = array(
		'NEW'=>'state_01.png'
		, 'START'=>'state_02.png'
		, 'HALF'=>'state_03.png'
		, 'DONE'=>'state_04.png'
		, 'CANCELED'=>'state_05.png'
		, 'DELEGATED'=>'state_06.png'
		, 'IMPORTANT'=>'state_07.png'
		, 'ATTENTION'=>'state_08.png'
	);
    
	/**
	 * Конструктор
	 */
    function __construct() {
        parent::Controller();
    }

	/**
	 * Этот метод вызывается до любого другого метода контроллера.
	 * В нем проверяется прошел пользователь авторизации или нет.
	 * И соответственно ограничивается доступ к приложению.
	 * @param object $method имя вызванного метода
	 * @return 
	 */
    function _remap($method) {
        //для не авторизовавшегося пользователя доступ закрыт ко всем методам
        //кроме login и checklogin
        if ($method == 'login' || $method == 'checklogin' || $this->redux_auth->logged_in()) {
            $this->$method();
        }
        else {
        	//если авторизациия не пройдена - редирект на страницу входа
            redirect('main/login');
        }
    }
    
    /**
     * Создает страницу с формой входа
     */
    function login() {
    	//если вход выполнени - редирект на страницу со списком записей
        if ($this->redux_auth->logged_in()) {
            redirect('bugtracker/page');
        }
        
		//формируем страницу
        $pageData['title'] = lang('title_login');
        
        $this->load->helper('form');
        
        $this->load->view('header', $pageData);
        $this->load->view('messageblock');
        $this->load->view('loginform');
        $this->load->view('footer');
    }
    
    /**
     * Проверяет имя пользователя и пароль. Если соответствующий пользователь
     * найден - выполняется переход на главную страницу приложения
     * (main/index), в противном случае - выводим сообщения об ошибке.
     */
    function checklogin() {
        $this->load->helper('form');
		//настраиваем библиотеку form_validation
        $this->load->library('form_validation');
		$this->form_validation->set_error_delimiters('<p class="error">', '</p>');
        $this->form_validation->set_rules('u_name', 'lang:login_u_name', 'required');
        $this->form_validation->set_rules('u_pass', 'lang:login_u_pass', 'required');
		//проверяем полученные данные
        if ($this->form_validation->run() === FALSE) {
            //форма заполнена не правильно (не введены имя или пароль)
            $pageData['title'] = lang('title_login_err');
            
            $this->load->view('header', $pageData);
            $this->load->view('messageblock');
            $this->load->view('loginform');
            $this->load->view('footer');
        }
        else {
            //форма заполнена правильно
            $login    = $this->input->post('u_name');
            $password = $this->input->post('u_pass');
            
			//проверяем соответствие введенного имени и пароля
            if ($this->redux_auth->login($login, $password)) {
                //пользователь найден
                $this->session->set_flashdata('message', lang('login_hi').$login);
                redirect('main/index');
            }
            else {
                //пользователь не найден
                $this->session->set_flashdata('message', lang('login_err'));
                redirect('main/login');
            }
        }
    }
    
    /**
     * Выход из приложения и редирект на страницу main/login.
     */
    function logout() {
        $this->redux_auth->logout();
        redirect('main/login');
    }
    
    /**
     * Главная страница приложения.
     */
    function index() {
        $this->load->model('mtodonotes');
        
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getAllNotes();
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer');
    }
    
	/**
	 * На этой странице будет показан список не завершенных задач, т.е.
	 * с состоянием NEW, START, HALF, IMPORTANT или ATTENTION.
	 * @return 
	 */
	function uncomplete() {
		$this->load->model('mtodonotes');
		
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getUncomplete();
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer');
	}
	
	/**
	 * На этой странице будет показан список важных задач, т.е.
	 * с состояниями IMPORTANT или ATTENTION.
	 * @return 
	 */
	function important() {
		$this->load->model('mtodonotes');
		
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getImportant();
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer');
	}
	
	/**
	 * На этой странице будет показан список задач с сегодняшней датой.
	 * @return 
	 */
	function fortoday() {
		$this->load->model('mtodonotes');
		
        $pageData['title'] = lang('title_main');
		//получаем список записей
        $pageData['notes'] = $this->mtodonotes->getForToday();
        
        $this->load->helper('form');
        
		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
        $this->load->view('todolist');
        $this->load->view('footer');
	}
	
	/**
	 * На этой странице размещена форма, позволяющая фильтровать задачи.
	 * @return 
	 */
    function filters() {
        $pageData['title'] = lang('title_filters');
		$pageData['searchDone'] = FALSE;
		$pageData['hasResults'] = FALSE;

	    $this->load->helper('form');
		//настраиваем form_validation
	    $this->load->library('form_validation');
		//эти правила нужно установить для того, чтобы работало восстановление значений для чекбоксов
	    $this->form_validation->set_rules('state[]', 'NEW');
	    $this->form_validation->set_rules('state[]', 'START');
	    $this->form_validation->set_rules('state[]', 'HALF');
	    $this->form_validation->set_rules('state[]', 'DONE');
	    $this->form_validation->set_rules('state[]', 'CANCELED');
	    $this->form_validation->set_rules('state[]', 'DELEGATED');
	    $this->form_validation->set_rules('state[]', 'IMPORTANT');
	    $this->form_validation->set_rules('state[]', 'ATTENTION');
	    $this->form_validation->set_rules('from_date', 'lang:from_date', 'required|callback_date_check');
	    $this->form_validation->set_rules('to_date', 'lang:to_date', 'required|callback_date_check');
		//проверяем полученные данные
	    if ($this->form_validation->run() === FALSE) {
	        //ошибка
			//описания ошибок будут выведены около соответствующих полей
	    } else {
	        //форма заполнена правильно
			//формируем массив с данными для поиска
			$searchData['states'] = $this->input->post('state');
			$searchData['from'] = $this->input->post('from_date');
			$searchData['to'] = $this->input->post('to_date');
			
			$this->load->model('mtodonotes');
			//запускаем поиск
	        $pageData['notes'] = $this->mtodonotes->find($searchData);
			//в зависимости от результатов устанавливаем hasResults и searchDone
			if ($pageData['notes'] !== FALSE) {
				$pageData['hasResults'] = TRUE;
			}
			if ($this->input->post('filterSbm')) {
				$pageData['searchDone'] = TRUE;
			}
	    }

		//формируем страницу
        $this->load->view('header', $pageData);
        $this->load->view('mainmenu');
        $this->load->view('messageblock');
		$this->load->view('filter.php');
		//показываем результаты поиска если они есть
		if ($pageData['hasResults']) {
	        $this->load->view('todolist');
		}
        $this->load->view('footer');
	}
	
	/**
	 * AJAX метод. Изменяет состояние записи.
	 * Параметры (передаются в массиве $_POST):
	 * n_id - id записи, которую нужно изменить
	 * n_state - новое состояние (передается ключ массива $states)
	 * @return 
	 */
    function updatestate() {
	    $res = array();
	       
		//настраиваем form_validation
	    $this->load->library('form_validation');
	        
	    $this->form_validation->set_rules('n_id', 'lang:n_id', 'required|integer');
	    $this->form_validation->set_rules('n_state', 'lang:n_state', 'required');
	        
	    if ($this->form_validation->run() === FALSE) {
	        //ошибка
	        $res['status'] = 'ERR';
	        $res['message'] = validation_errors();
	    } else {
	        //форма заполнена правильно
	        $id = $this->input->post('n_id');
			$noteData['td_state'] = $this->input->post('n_state');
	        //сохраняем запись
	        $this->load->model('mtodonotes');
	        if ($this->mtodonotes->updateNote($id, $noteData)) {
	            $res['status'] = 'OK';
	            $res['id'] = $id;
				$res['new_state'] = $this->states[$noteData['td_state']];
	        } else {
	            $res['status'] = 'ERR';
	            $res['message'] = lang('update_db_err');
	        }
	    }
	    //возвращаем результат    
	    echo json_encode($res);
	}

	/**
	 * AJAX метод. Изменяет текст записи.
	 * Параметры (передаются в массиве $_POST):
	 * id - id блока на странице в котором находится запись, которую нужно изменить
	 * (id блоков не могут начинаться с цифры, поэтому чтобы получить id записи нужно
	 * из id блока вырезать приставку (note_text_))
	 * value - новый текст
	 * @return 
	 */
    function updatetext() {
    	//получаем параметры
	    $id = $this->input->post('id');
	    $text = $this->input->post('value');
	    
		//получаем id записи
	    if ($id !== FALSE) {
	        $id = substr($id, 10);
	    } else {
	        echo lang('update_id_err');
	        return;
	    }
	        
	    if ($text === FALSE || $text == '') {
	        echo lang('update_text_err');
	        return;
	    }
	    
		//изменяем запись в БД
	    $this->load->model('mtodonotes');
	    if ($this->mtodonotes->updateNote($id, array('td_text'=>$text))) {
	    	//возвращаем измененный текст
	        echo $text;
	    } else {
	    	//или сообщение об ошибке
	        echo lang('update_db_err');
	    }
	}

	/**
	 * AJAX метод. Изменяет дату записи.
	 * Параметры (передаются в массиве $_POST):
	 * id - id блока на странице в котором находится запись, которую нужно изменить
	 * (id блоков не могут начинаться с цифры, поэтому чтобы получить id записи нужно
	 * из id блока вырезать приставку (note_date_))
	 * value - новая дата
	 * @return 
	 */
	function updatedate() {
	    $id = $this->input->post('id');
	    $date = $this->input->post('value');
	        
	    if ($id !== FALSE) {
	        $id = substr($id, 10);
	    } else {
	        echo lang('update_id_err');
	        return;
	    }
	        
	    if ($date === FALSE || preg_match('/^\d{4}-\d{2}-\d{2}$/', $date) != 1) {
	        echo lang('update_date_err');
	        return;
	    }
	        
		//сохраняем новую дату в базе
	    $this->load->model('mtodonotes');
	    if ($this->mtodonotes->updateNote($id, array('td_date'=>$date))) {
	    	//и возвращаем измененную дату
	        echo $date;
	    } else {
	    	//или сообщение об ошибке
	        echo lang('update_db_err');
	    }
	}
    
    /**
	 * AJAX метод. Сохраняет новую запись в базе.
	 * Параметры (передаются в массиве $_POST):
	 * n_date - дата новой записи
	 * n_text - текст записи
     */
    function addnote() {
        
        $res = array();
        
		//настраиваем библиотеку form_validation
        $this->load->library('form_validation');
        $this->form_validation->set_rules('n_text', 'lang:n_text', 'required');
        $this->form_validation->set_rules('n_date', 'lang:n_date', 'required|exact_length[10]|callback_date_check');
        if ($this->form_validation->run() === FALSE) {
            //ошибка
            $res['status'] = 'ERR';
            $res['message'] = validation_errors();
        }
        else {
            //форма заполнена правильно
            //получаем введенные данные...
            $noteData['td_text'] = $this->input->post('n_text');
            $noteData['td_date'] = $this->input->post('n_date');
                
            //...и сохраняем запись
            $this->load->model('mtodonotes');
            $noteData['td_state'] = 'NEW'; // состояние новой записи всегда NEW
            if (($id = $this->mtodonotes->addNote($noteData)) !== FALSE) {
                $res['status'] = 'OK';
				$res['id'] = $id;
				$res['state_img_url'] = base_url().'images/state_01.png';
				$res['state'] = $noteData['td_state'];
				$res['text'] = $noteData['td_text'];
				$res['date'] = $noteData['td_date'];
            }
            else {
                $res['status'] = 'ERR';
                $res['message'] = lang('add_db_err');
            }
        }
        //возвращаем результат
        echo json_encode($res);
    }
	
	/**
	 * AJAX метод. Изменяет удаляет запись.
	 * Параметры (передаются в массиве $_POST):
	 * n_id - id записи, которую нужно удалить
	 * @return JSON строка с описанием результа операции или ошибки
	 */
	function removenote() {
		$res = array();
		
		//настраиваем form_validation
		$this->load->library('form_validation');
		
		$this->form_validation->set_rules('n_id', 'lang:n_id', 'required|integer');
		
        if ($this->form_validation->run() === FALSE) {
            $res['status'] = 'ERR';
            $res['message'] = validation_errors();
        }
        else {
        	$id = $this->input->post('n_id');
        	//удаляем запись
			$this->load->model('mtodonotes');
			if ($this->mtodonotes->removenote($id)) {
				$res['status'] = 'OK';
				$res['id'] = $id;
			}
			else {
				$res['status'] = 'ERR';
				$res['message'] = lang('remove_db_err');
			}
		}
		//возвращаем результат
		echo json_encode($res);
	}
	
	/**
	 * Этот метод выполняет проверку формата даты
	 * @param object $str строка с датой
	 * @return TRUE - если дата записана правильно, FALSE - в противном случае
	 */
	function date_check($str) {
		if (preg_match('/^\d{4}-\d{2}-\d{2}$/', $str) == 1) {
			return TRUE;
		}
		else {
			$this->form_validation->set_message('date_check', lang('add_date_err'));
			return FALSE;
		}
	}
}

// end of main.php