<?php

class MToDoNotes extends Model {
    
    function __construct() {
        parent::Model();
    }
    
    /**
     * Ищет все записи.
     * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
     */
    function getAllNotes() {
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
    }

	/**
	 * Ищет все не завершенные задачи, т.е. с состоянием
	 * NEW, START, HALF, IMPORTANT или ATTENTION.
	 * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
	 */
	function getUncomplete() {
		$this->db->where('td_state', 'NEW');
		$this->db->or_where('td_state', 'START');
		$this->db->or_where('td_state', 'HALF');
		$this->db->or_where('td_state', 'IMPORTANT');
		$this->db->or_where('td_state', 'ATTENTION');
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
	}
	
	/**
	 * Ищет все важные задачи, т.е. с состоянием
	 * IMPORTANT или ATTENTION.
	 * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
	 */
	function getImportant() {
		$this->db->where('td_state', 'IMPORTANT');
		$this->db->or_where('td_state', 'ATTENTION');
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
	}
	
	/**
	 * Ищет все задачи с сегодняшней датой.
	 * @return объект, содержащий все найденные записи,
     * если записи отсутствуют - false.
	 */
	function getForToday() {
		$this->db->where('td_date = CURDATE()');
        $query = $this->db->get('todonotes');
        if ($query->num_rows() > 0) {
            return $query->result();
        }
        else {
            return false;
        }
	}
    
	/**
	 * Сохраняет новую задачу в БД. 
	 * @param object $noteData массив с данными записи, формируется контроллером.
	 * @return id новой записи (если она создана), false - в случае возникновения ошибки.
	 */
    function addNote($noteData) {
        if ($this->db->insert('todonotes', $noteData)) {
        	return $this->db->insert_id();
        }
		return FALSE;
    }
	
	/**
	 * Изменяет существующую запись в БД
	 * @param object $id id записи, которую нужно изменить
	 * @param object $noteData массив с данными записи, формируется контроллером
	 * @return true - если операция успешно выполнена, false - в противном случае
	 */
	function updateNote($id, $noteData) {
		$this->db->where('id', $id);
		return $this->db->update('todonotes', $noteData);
	}
	
	/**
	 * Удаляет запись из БД
	 * @param object $id id записи, которую нужно удалить
	 * @return true - если операция успешно выполнена, false - в противном случае
	 */
	function removeNote($id) {
		$this->db->where('id', $id);
		return $this->db->delete('todonotes');
	}
	
	/**
	 * Ищет записи в БД по заданным параметрам
	 * @param object $searchData массив с параметрами, формируется контроллером на
	 * основе формы, которую заполнил пользователь
	 * @return объект, содержащий все найденные записи,
     * если записи не найдены - false.
	 */
	function find($searchData) {
		if ($searchData['states'] !== FALSE) {
			$this->db->where_in('td_state', $searchData['states']);
		}
		$this->db->where('td_date >=', $searchData['from']);
		$this->db->where('td_date <=', $searchData['to']);
		$query = $this->db->get('todonotes');
		if ($query->num_rows() > 0) {
			return $query->result();
		}
		else {
			return FALSE;
		}
	}
}

// end of mtodonotes.php